/* Program to format font TILES to H-P LaserJet2 soft font (portrait)        */

/* Coded in Microsoft C 5.1 by Daniel Ross, starting 3/11/88                 */
/* Revised 1/23/89    09:40                                                  */

/* Although not explicitly stated in any H-P manual, this code is written    */
/* with the assumption that the MSB of a word is sent before the LSB.        */

#include <STDLIB.H>
#include <STDIO.H>
#include <STRING.H>
#include <MATH.H>
/* More files are included below                                             */

#define TILESColsPerChar         31
#define TILESSpaceCols           16
#define TILESRowsAboveBaseLine   21
#define TILESDescenderRows       0
#define TILESRowsPerChar (TILESRowsAboveBaseLine + TILESDescenderRows)
#define ColsSeparatingChars      4

long   TILES [128] [TILESRowsPerChar]   =
  {
#include "TILES.C"
  } ;

#define FALSE               0
#define TRUE                1
#define TorF                int

#define Escape              0x1B

int     CurrentCharIndex ;
long    CurrentCharPattern [TILESRowsPerChar] ;
int     BlankRowsOnTop ;
int     BlankRowsOnBottom ;
int     BlankColsOnRight ;
int     PatternWidth_Dots ;
int     PatternWidth_Bytes ;
int     PatternHeight ;
int     TotalPatternBytes ;
FILE    * FileHandle ;
char    FileName []   =   "TILES.SFP" ;

void   CopyToCurrentCharPattern (void)
  {
    static int   RowIndex ;
    for (RowIndex = 0 ;   RowIndex < TILESRowsPerChar ;   ++ RowIndex)
      CurrentCharPattern [RowIndex]   =
        TILES [CurrentCharIndex] [RowIndex];
  }

void   CalcBlankRowsOnTop (void)
  {
    for
      (
        BlankRowsOnTop = 0
          ;
        BlankRowsOnTop < TILESRowsPerChar
          ;
        ++ BlankRowsOnTop
      )
      {
        if (CurrentCharPattern [BlankRowsOnTop])   return ;
      } ;
  }

void   CalcBlankRowsOnBottom (void)
  {
    for
      (
        BlankRowsOnBottom = 0
          ;
        BlankRowsOnBottom < TILESRowsPerChar
          ;
        ++ BlankRowsOnBottom
      )
      {
        if (CurrentCharPattern [(TILESRowsPerChar - 1) - BlankRowsOnBottom])
          return ;
      } ;
  }

void   CalcBlankColsOnRight (void)
  {
    static int    RowIndex ;
    static long   Mask ;
    for
      (
        BlankColsOnRight = 0
          ;
        BlankColsOnRight < TILESColsPerChar
          ;
        ++ BlankColsOnRight
      )
      {
        Mask   =   0x80000000 >> (31 - BlankColsOnRight) ;
        for (RowIndex = 0 ;   RowIndex < TILESRowsPerChar ;   ++ RowIndex)
          {
            if (CurrentCharPattern [RowIndex] & Mask)   return ;
          } ;
      } ;
  }

void   JustifyTheCurrentCharPattern (void)
  {
    static int   RowIndex ;
    if (BlankRowsOnTop)
      {
        for
          (
            RowIndex = 0
              ;
            RowIndex < (TILESRowsPerChar - BlankRowsOnTop)
              ;
            ++ RowIndex
          )
          CurrentCharPattern [RowIndex]   =
            CurrentCharPattern [RowIndex + BlankRowsOnTop] ;
      } ;
  }

void   CalcPatternSizeToSend (void)
  {
    PatternWidth_Dots     =   4 * (32 - BlankColsOnRight) ;
    PatternWidth_Bytes    =   (PatternWidth_Dots + 7) / 8 ;
    PatternHeight         =
      4 * (TILESRowsPerChar - BlankRowsOnTop - BlankRowsOnBottom) ;
    TotalPatternBytes     =   PatternWidth_Bytes * PatternHeight ;
  }

/* Make 4 copies of each bit                                                 */
int   TwoBitsToByte (int Given2Bits)
  {
    static int   TwoBitPosn, ResultByte ;
    ResultByte = 0 ;
    for (TwoBitPosn = 1 ;   TwoBitPosn >= 0 ;   -- TwoBitPosn)
      {
        if (Given2Bits & (1 << TwoBitPosn))
          ResultByte   |=
            (1 << ((4 * TwoBitPosn) + 0))
              |
            (1 << ((4 * TwoBitPosn) + 1))
              |
            (1 << ((4 * TwoBitPosn) + 2))
              |
            (1 << ((4 * TwoBitPosn) + 3)) ;
      } ;
    return ResultByte ;
  }

void   Print1Byte (int TheByte)
  {
    fprintf (FileHandle, "%c", TheByte & 0xFF) ;
  }

void   Print1Word (int TheWord)
  {
    Print1Byte (TheWord >> 8) ;
    Print1Byte (TheWord) ;
  }

void   PrintTheFontDescriptor (void)
  {
    Print1Word (64) ;                   /* Font descriptor size              */
    Print1Word (0) ;                    /* Reserved byte & font type         */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word ((4 * TILESRowsAboveBaseLine) - 1) ;
    Print1Word (4 * TILESColsPerChar) ;
    Print1Word (4 * TILESRowsPerChar) ;
    Print1Word (1) ;                    /* Orientation & spacing             */
    Print1Word
      ((unsigned int) (((long) 2047 * (long) 32) + (long) 65 - (long) 64)) ;
                                        /* Symbol set                        */
    Print1Word (4 * (TILESSpaceCols + ColsSeparatingChars) * 4) ;  /* Pitch  */
    Print1Word (4 * TILESRowsPerChar * 4) ;                        /* Height */
    Print1Word (56 * 4) ;               /* x height                          */
    Print1Word (0) ;                    /* Width type & style                */
    Print1Word (0) ;                    /* Stroke weight & typeface          */
    Print1Word (0) ;                    /* Reserved byte & serif style       */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (((-6)<<8) + 3) ;        /* Underline distance & height       */
    Print1Word (100 * 4) ;              /* Text height                       */
    Print1Word (4 * (TILESColsPerChar + ColsSeparatingChars) * 4); /* T width*/
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Pitch & height extended           */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    fprintf (FileHandle, "%16s", "Tiles           ") ;
  }

void   main (void)
  {
    static int      RowIndex, ByteIndex ;
    static long     CurrentLong ;
    if ( ! (FileHandle = fopen (FileName, "wb")))
      {
        printf ("\nUnable to open font file %s", FileName) ;
        exit (1) ;
      } ;
    /* Print the command that precedes the font descriptor                   */
    fprintf (FileHandle, "%c)s64W", Escape) ;
    PrintTheFontDescriptor () ;
    for
      (
        CurrentCharIndex =  '!'
          ;
        CurrentCharIndex != 128
          ;
        ++ CurrentCharIndex
      )
      {
        /* Print the command that selects the character                      */
        fprintf (FileHandle, "%c*c%dE", Escape, CurrentCharIndex) ;
        CopyToCurrentCharPattern        () ;
        CalcBlankRowsOnTop              () ;
        CalcBlankRowsOnBottom           () ;
        CalcBlankColsOnRight            () ;
        JustifyTheCurrentCharPattern    () ;
        CalcPatternSizeToSend           () ;
        /* Print the command that precedes the character descriptor          */
        fprintf
          (
            FileHandle
              ,
            "%c(s%dW"
              ,
            Escape
              ,
            16 + TotalPatternBytes
          ) ;
        /* Print the character descriptor                                    */
        Print1Word ((4<<8) + 0) ;       /* Format & continuation             */
        Print1Word ((14<<8) + 1) ;      /* Descriptor size & class           */
        Print1Word (0) ;                /* Orientation & reserved byte       */
        Print1Word (0) ;                /* Left offset                       */
        Print1Word ((4 * (TILESRowsAboveBaseLine - BlankRowsOnTop)) - 1) ;
        Print1Word (PatternWidth_Dots) ;
        Print1Word (PatternHeight) ;
        Print1Word ((PatternWidth_Dots + (4 * ColsSeparatingChars)) * 4) ;
                                        /* dX                                */
        for (RowIndex = 0 ;   RowIndex != PatternHeight ;   ++ RowIndex)
          {
            CurrentLong   =   CurrentCharPattern [RowIndex / 4] ;
            for
              (
                ByteIndex =  0
                  ;
                ByteIndex != PatternWidth_Bytes
                  ;
                ++ ByteIndex
              )
              Print1Byte
               (TwoBitsToByte ((int) (CurrentLong >> (30 - (ByteIndex * 2)))));
          } ;
      } ;
    if (fclose (FileHandle))
      {
        printf ("\nError closing font file %s", FileName) ;
        exit (1) ;
      } ;
    exit (0) ;
  }

/* End of file                                                              */
